import uuid
import datetime
from typing import Optional

import click

from gcs_cli import version
from gcs_cli.helpers import lookup_project, requires_login
from gcs_cli.parsing import globus_command
from gcs_cli.prettyprint import FORMAT_TEXT_RECORD, formatted_print
from gcs_cli.sdk_client.auth_client import AuthClient, MissingConsent
from gcs_cli.sdk_client.data import (
    AuthCredentialDocument,
    AuthClientDocument,
    AuthScopeDocument,
)
from gcs_cli.services.auth import get_client
from globus_sdk import BasicAuthorizer, scopes


@globus_command(
    "create",
    short_help="Register a new GCS Auth client",
    disable_options=["explicit_host"],
)
@click.option(
    "--project-id",
    metavar="PROJECT_ID",
    type=click.UUID,
    help="Auth project ID where the new authentication policy will be stored",
)
@click.argument("DISPLAY_NAME", type=click.STRING)
@requires_login
def client_create(
    project_id: Optional[uuid.UUID],
    display_name: str,
) -> None:
    """
    Register a new GCS Auth client.
    """
    client = get_client()

    # If the caller did not specify a project ID, we are going to lookup projects
    # owned by the caller and, if there is only a single project, we will use it.
    if project_id is None:
        project_id = lookup_project(client, "--project-id")

    #
    # Create the Auth client
    #
    client_doc = AuthClientDocument(
        project=project_id,
        name=display_name,
        public_client=False,
        client_type="globus_connect_server",
    )

    try:
        create_client_res = client.create_client(client_doc)
    except MissingConsent as e:
        raise click.ClickException(
            "Missing consent for the policy API. Please run the following "
            "command:\n"
            f"    globus-connect-server session consent {e.scope}"
        )
    client_id: str = create_client_res["id"]

    #
    # Create a client credential
    #
    name = f"Created by {version.app_name} at {datetime.datetime.now()}"
    credential_doc = AuthCredentialDocument(name=name)
    create_credential_res = client.create_credential(client_id, data=credential_doc)
    client_secret = create_credential_res["secret"]

    #
    # The following kludge is to workaround Auth bug SC-23300 which forces us to use
    # a non-token AuthClient for the remainder of the operations.
    # Once that bug is fixed, we can remove this and use 'client' from above.
    #
    client2 = AuthClient(
        authorizer=BasicAuthorizer(client_id, client_secret), app_name=version.app_name
    )
    #
    # End kludge block.
    #

    #
    # Look up dependent scopes.
    #
    dependent_scopes = [
        scopes.AuthScopes.view_identities,
        scopes.AuthScopes.openid,
        scopes.AuthScopes.email,
        scopes.AuthScopes.profile,
        scopes.GroupsScopes.view_my_groups_and_memberships,
        scopes.TransferScopes.all,
    ]

    get_scopes_res = client2.get_scopes(dependent_scopes)

    #
    # Use our new client to create our scope(s)
    #
    scope_doc = AuthScopeDocument(
        name=f"Manage collections on {display_name}",
        description=(
            "Access to all capabilities of the Globus Connect Server. This includes "
            "creation, deletion and management of collections and storage system "
            f"credentials on the server {display_name}"
        ),
        scope_suffix="manage_collections",
        advertised=False,
        allow_refresh_tokens=True,
        dependent_scopes=[
            {"scope": scope["id"], "optional": False, "requires_refresh_token": False}
            for scope in get_scopes_res["scopes"]
        ],
    )

    # We can ignore the response
    client.create_scope(client_id, scope_doc)

    formatted_print(
        {**create_client_res.data, "secret": client_secret},
        text_format=FORMAT_TEXT_RECORD,
        fields=[
            ("Display Name", "name"),
            ("ID", "id"),
            ("Secret", "secret"),
        ],
    )
