from typing import Generator, Iterable, Mapping

import click

from gcs_cli.helpers import requires_login
from gcs_cli.parsing import globus_command
from gcs_cli.prettyprint import FORMAT_TEXT_TABLE, formatted_print
from gcs_cli.sdk_client.auth_client import MissingConsent
from gcs_cli.services.auth import get_client


@globus_command(
    "list",
    short_help="List existing GCS clients owned by you",
    disable_options=["explicit_host"],
)
@requires_login
def client_list() -> None:
    """
    Print Auth clients owned by the user. When we print in table format, only
    display clients with type 'globus_connect_server'.
    """
    client = get_client()
    try:
        res = client.get_client_list()
    except MissingConsent as e:
        raise click.ClickException(
            "Missing consent for the policy API. Please run the following command:\n"
            f"    globus-connect-server session consent {e.scope}"
        )

    def _sort_callback(
        client_resources: Iterable[Mapping],
    ) -> Generator[Mapping, None, None]:
        for cr in client_resources:
            if cr["client_type"] == "globus_connect_server":
                yield cr

    formatted_print(
        res,
        text_format=FORMAT_TEXT_TABLE,
        fields=[
            ("ID", "id"),
            ("Display Name", "name"),
        ],
        sort_callback=_sort_callback,
    )
